/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "domstubs.idl"

interface nsIDOMNodeFilter;

[uuid(a86bdac9-ff89-4c94-8160-4fe86733bab8)]
// Introduced in DOM Level 2, updated to DOM Level 4:
interface nsIDOMNodeIterator : nsISupports
{
  readonly attribute nsIDOMNode       root;
  readonly attribute unsigned long    whatToShow;
  readonly attribute nsIDOMNodeFilter filter;
  nsIDOMNode         nextNode()
                                        raises(DOMException);
  nsIDOMNode         previousNode()
                                        raises(DOMException);
  void               detach();

  // WebKit extensions, convenient for debugging.
  readonly attribute nsIDOMNode referenceNode;
  readonly attribute boolean    pointerBeforeReferenceNode;
};
